module HulkHogan
  class HulkHogan

    # You may initialize you player but the the initialize method must take NO paramters.
    # The player will only be instantiated once, and will play many games.
    def initialize
    end

    # Before starting a game, this method will be called to inform the player of all the possible words that may be
    # played.
    def word_list=(list)
      @words = list
    end

    # a new game has started.  The number of guesses the player has left is passed in (default 6),
    # in case you want to keep track of it.
    def new_game(guesses_left)
      @word_list = @words
      @guesses = []
    end

    # Each turn your player must make a guess.  The word will be a bunch of underscores, one for each letter in the word.
    # after your first turn, correct guesses will appear in the word parameter.  If the word was "shoes" and you guessed "s",
    # the word parameter would be "s___s", if you then guess 'o', the next turn it would be "s_o_s", and so on.
    # guesses_left is how many guesses you have left before your player is hung.
    def guess(word, guesses_left)
      update_word_list(word)
      update_frequencies
      guess_letter
    end

    # notifies you that your last guess was incorrect, and passes your guess back to the method
    def incorrect_guess(guess)
      @word_list = @word_list.select {|w| ! w.include?(guess)}
    end

    # notifies you that your last guess was correct, and passes your guess back to the method
    def correct_guess(guess)
      @word_list = @word_list.select {|w| w.include?(guess)}
      @guesses << guess
    end

    # you lost the game.  The reason is in the reason parameter
    def fail(reason)
    end

    # The result of the game, it'll be one of 'win', 'loss', or 'fail'.
    # The spelled out word will be provided regardless of the result.
    def game_result(result, word)
      words = @words.dup
      words.delete word
      @words = words
    end
    
    protected
    def update_word_list(word)
      @word_list = @word_list.select {|w| w =~ filter_pattern(word)}
    end
    
    def filter_pattern(word)
      Regexp.new("^" + word.gsub("_", "\\w") + "$")
    end
    
    def update_frequencies
      @letter_frequencies = get_frequencies.sort_by {|k, v| [v,k]}
    end
    
    def get_frequencies
      result = {}
      @word_list.each {|w| add_unique_letters(w,result)}
      result
    end
    
    def add_unique_letters(word, result)
      normalize(word).each do |c|
        result[c] = result[c].to_i + 1
      end
    end
    
    def normalize(word)
      word.split('').uniq
    end
    
    def guess_letter
      g = @letter_frequencies.last[0]
      while @guesses.include?(g) do
        g = @letter_frequencies.shift[0]
      end
      g
    end
  end
end