require File.expand_path(File.dirname(__FILE__) + "/../spec_helper")
require 'hulk_hogan/hulk_hogan'

describe HulkHogan::HulkHogan do
  before do
    @hulk = HulkHogan::HulkHogan.new
  end

  it "should be instantiable with no paramters" do

    lambda { HulkHogan::HulkHogan.new }.should_not raise_error

  end
  
  it "should store the words" do
    @hulk.word_list = %w(ciao bambina)
    @hulk.new_game(6)
    @hulk.instance_variable_get(:@word_list).should == %w(ciao bambina)
  end

  it "should filter the words on guess based on length" do
    @hulk.word_list = %w(a b c aa bb cc aaa bbb ccc aaaa bbbb cccc dddd)
    @hulk.new_game(6)
    @hulk.guess('___', 6)
    @hulk.instance_variable_get(:@word_list).should == %w(aaa bbb ccc)    
  end
  
  it "should filter the words on guess based on words" do
    @hulk.word_list = %w(abc bcd cde def efg fgh ghi hij)
    @hulk.new_game(6)
    @hulk.guess('___', 6)
    @hulk.instance_variable_get(:@word_list).size.should == 8
    @hulk.guess('_c_', 6)
    @hulk.instance_variable_get(:@word_list).size.should == 1
  end

  it "should set the words frequency on guess" do 
    @hulk.word_list = %w(ciao pilu iiii)
    @hulk.new_game(6)
    @hulk.guess('____', 6)
    @hulk.instance_variable_get(:@letter_frequencies).should == [['a', 1], ['c', 1], ['l', 1], ['o', 1], ['p', 1], ['u', 1], ['i', 3]]
  end
  
  it "should guess the most probable letter" do
    @hulk.word_list = %w(ciao pilu)
    @hulk.new_game(6)
    @hulk.guess('____', 6).should == 'i'
  end
  
  it "should remove the used letter on incorrect guess" do
    @hulk.word_list = %w(ciao pilu ciii ciii ciii cono)
    @hulk.new_game(6)
    @hulk.incorrect_guess("i")
    @hulk.instance_variable_get(:@word_list).should == ["cono"]
  end

  it "should keep track of guesses" do
    @hulk.word_list = %w(ciao pilu ciii ciii ciii cono)
    @hulk.new_game(6)
    @hulk.correct_guess("i")
    @hulk.instance_variable_get(:@word_list).should_not include("cono")
    @hulk.guess('c___', 6).should_not == 'i'
  end
  
  it "should not crash on consecutive games" do
    h = HulkHogan::HulkHogan.new
    h.word_list = %w(liz pilu giovanni ciao mamma come stai alfabeto megera cane pane salame standard ruccia roccia reccia riccia rapalo ropolo romolo)
    h.new_game(6)
    6.times {|i| h.incorrect_guess('z')}
  end
  
  it "should set the word list on new_game" do
    @hulk.word_list = %w(ciao pilu ciii ciii ciii cono)
    @hulk.new_game(6)
    @hulk.incorrect_guess(@hulk.guess('____', 6))
    @hulk.new_game(6)
    @hulk.instance_variable_get(:@word_list).should == %w(ciao pilu ciii ciii ciii cono)
  end
  
  it "should remove already used words after game finishes" do
    @hulk.word_list = %w(ciao pilu mamma cane pane)
    @hulk.new_game(6)
    @hulk.game_result('fail', 'ciao')
    @hulk.instance_variable_get(:@words).should == %w(pilu mamma cane pane)
  end
end

describe HulkHogan::HulkHogan, "filter_pattern" do
  it "should filter an underscore pattern" do
    @hulk = HulkHogan::HulkHogan.new
    @hulk.send(:filter_pattern, "c____o").should == /^c\w\w\w\wo$/
    @hulk.send(:filter_pattern, "ce___o").should == /^ce\w\w\wo$/
  end
end